# Copyright (C) 2013-2023, Hiroshi Izumi (izumi.h@aist.go.jp)
# This code is part of the Biopython distribution and governed by its
# license.  Please see the LICENSE file that should have been included
# as part of this package.

import os
import wx
import wx.stc
import shutil
from numpy import *
from itertools import groupby
from ast import literal_eval
from difflib import *

class MenuExample(wx.Frame):
    def __init__(self, parent, id, title):
        wx.Frame.__init__(self, parent, id, title, size=(1000, 325))

        menubar = wx.MenuBar()
        file = wx.Menu()
        #file.Append(-1, '&SDF_File_Open')
        #file.AppendSeparator()
        file.Append(10, '&Open')
        file.AppendSeparator()        
        file.Append(16, '&PDB_MMCIF_GZ_to_SSSC')
        file.AppendSeparator()        
        file.Append(11, '&PDB_to_SSSC')
        file.AppendSeparator()
        file.Append(14, '&SSSC_to_docx')
        file.AppendSeparator()
        file.Append(12, '&T_Characterization')
        file.AppendSeparator()
        file.Append(13, '&Alignment_Input')
        file.AppendSeparator()
        file.Append(15, '&Fuzzy_Structural_Homology')
        file.AppendSeparator()
        quit = wx.Menu()
        quit = wx.MenuItem(file, 1, '&Quit\tCtrl+Q')
        file.Append(quit)
        #file.AppendItem(quit)

        #self.Bind(wx.EVT_MENU, self.OnOpen, id=-1)
        self.Bind(wx.EVT_MENU, self.OnOpen1, id=10)
        self.Bind(wx.EVT_MENU, self.OnPDB_MMCIF_GZ_to_ssc, id=16)
        self.Bind(wx.EVT_MENU, self.OnPDB_to_ssc, id=11)
        self.Bind(wx.EVT_MENU, self.OnSSSC_to_docx, id=14)        
        self.Bind(wx.EVT_MENU, self.OnT_characterization, id=12)
        self.Bind(wx.EVT_MENU, self.OnAlignment, id=13)
        self.Bind(wx.EVT_MENU, self.OnFuzzy_search, id=15)
        self.Bind(wx.EVT_MENU, self.OnQuit, id=1)

        menubar.Append(file, '&File')
        self.SetMenuBar(menubar)

        panel = wx.Panel(self, -1)

        font = wx.SystemSettings.GetFont(wx.SYS_SYSTEM_FONT)
        #font = wx.SystemSettings_GetFont(wx.SYS_SYSTEM_FONT)
        font.SetPointSize(10)
        font.SetFamily(wx.FONTFAMILY_MODERN)

        vbox = wx.BoxSizer(wx.VERTICAL)

        #hbox1 = wx.BoxSizer(wx.HORIZONTAL)
        #st1 = wx.StaticText(panel, -1, 'SDF File')
        #st1.SetFont(font)
        #hbox1.Add(st1, 0, wx.RIGHT, 8)
        #self.tc = wx.TextCtrl(panel, -1)
        #hbox1.Add(self.tc, 1)
        #vbox.Add(hbox1, 0, wx.EXPAND | wx.LEFT | wx.RIGHT | wx.TOP, 10)

        #vbox.Add((-1, 5))

        #hbox12 = wx.BoxSizer(wx.HORIZONTAL)
        #st12 = wx.StaticText(panel, -1, 'Chirality')
        #st12.SetFont(font)
        #hbox12.Add(st12, 0, wx.RIGHT, 8)
        #self.tc12 = wx.TextCtrl(panel, -1)
        #hbox12.Add(self.tc12, 1)
        #vbox.Add(hbox12, 0, wx.EXPAND | wx.LEFT | wx.RIGHT | wx.TOP, 10)

        #vbox.Add((-1, 5))
        
        #hbox13 = wx.BoxSizer(wx.HORIZONTAL)
        #st13 = wx.StaticText(panel, -1, 'Chirality [Lone Pair]')
        #st13.SetFont(font)
        #hbox13.Add(st13, 0, wx.RIGHT, 8)
        #self.tc13 = wx.TextCtrl(panel, -1)
        #hbox13.Add(self.tc13, 1)
        #vbox.Add(hbox13, 0, wx.EXPAND | wx.LEFT | wx.RIGHT | wx.TOP, 10)

        #vbox.Add((-1, 5))

        hbox14 = wx.BoxSizer(wx.HORIZONTAL)
        st14 = wx.StaticText(panel, -1, '')
        st14.SetFont(font)
        hbox14.Add(st14, 0, wx.RIGHT, 8)
        self.tc14 = wx.stc.StyledTextCtrl(panel, -1)
        self.tc14.StyleSetFont(0, font)
        #self.tc14.SetSelection(-1, -1)
        #self.tc14.SetSelBackground(1, wx.GREEN)
        hbox14.Add(self.tc14, 1)
        vbox.Add(hbox14, 0, wx.EXPAND | wx.LEFT | wx.RIGHT | wx.TOP, 10)

        vbox.Add((-1, 10))

        hbox12 = wx.BoxSizer(wx.HORIZONTAL)
        st12 = wx.StaticText(panel, -1, 'Fragment Search')
        st12.SetFont(font)
        hbox12.Add(st12, 0, wx.RIGHT, 8)
        self.tc12 = wx.TextCtrl(panel, -1)
        hbox12.Add(self.tc12, 1)
        vbox.Add(hbox12, 0, wx.EXPAND | wx.LEFT | wx.RIGHT | wx.TOP, 10)

        vbox.Add((-1, 5))
        
        hbox13 = wx.BoxSizer(wx.HORIZONTAL)
        st13 = wx.StaticText(panel, -1, 'File Count')
        st13.SetFont(font)
        hbox13.Add(st13, 0, wx.RIGHT, 8)
        self.tc13 = wx.TextCtrl(panel, -1)
        hbox13.Add(self.tc13, 1)
        vbox.Add(hbox13, 0, wx.EXPAND | wx.LEFT | wx.RIGHT | wx.TOP, 10)
        
        vbox.Add((-1, 10))
        
        hbox5 = wx.BoxSizer(wx.HORIZONTAL)
        btn1 = wx.Button(panel, -1, 'OK', size=(70, 30))
        hbox5.Add(btn1, 0)
        #btn1_2 = wx.Button(panel, -1, 'Tight', size=(70, 30))
        #hbox5.Add(btn1_2, 0, wx.LEFT | wx.BOTTOM, 5)
        #btn1_3 = wx.Button(panel, -1, 'Complete', size=(70, 30))
        #hbox5.Add(btn1_3, 0, wx.LEFT | wx.BOTTOM, 5)
        btn2 = wx.Button(panel, -1, 'Close', size=(70, 30))
        hbox5.Add(btn2, 0, wx.LEFT | wx.BOTTOM, 5)
        vbox.Add(hbox5, 0, wx.ALIGN_RIGHT | wx.RIGHT, 10)

        btn1.Bind(wx.EVT_BUTTON, self.OnSubmit1, id=-1)
        #btn1_2.Bind(wx.EVT_BUTTON, self.OnSubmit3, id=-1)
        #btn1_3.Bind(wx.EVT_BUTTON, self.OnSubmit4, id=-1)
        btn2.Bind(wx.EVT_BUTTON, self.OnQuit, id=-1)
        
        panel.SetSizer(vbox)
        self.Centre()
        self.Show(True)

        self.CreateStatusBar()
        self.SetStatusText("Supersecondary Structure Code View")
        self.GetStatusBar().SetBackgroundColour(None)

        self.M_pick = None
        
    def OnOpen1(self, e):
        self.dirName2 = ''
        dialog = wx.FileDialog(self, "Choose a file", self.dirName2, "", "*.*", wx.FD_OPEN)
        if dialog.ShowModal() == wx.ID_OK:
            self.fileName2 = dialog.GetFilename()
            self.dirName2 = dialog.GetDirectory()
            file = open(os.path.join(self.dirName2, self.fileName2), 'r')
            datab1 = file.readlines()
            sdf_fileName = os.path.join(self.dirName2, self.fileName2)
            file.close()
            seq = str(datab1[1])
            ssc = str(datab1[2])
            #ssc_l = list(ssc)
            self.tc14.Clear()
            self.tc14.AppendText(ssc)
            self.tc14.AppendText("\n")
            self.tc14.AppendText(seq)
            #for i, ele in enumerate(ssc_l):
                #if ele == "H":
                    #self.tc14.AppendText(ele)
                    #self.tc14.SetSelection(-1, -1)
                    #self.tc14.SetAdditionalSelBackground(wx.GREEN)
                    #self.tc14.ShowPosition(10)
                    #self.tc14.SetSelBackground(1, wx.GREEN)
                    #self.tc14.AddSelection(i, i+1)
                    #self.tc14.SetAdditionalSelBackground(wx.GREEN)
                #elif ele == "S":
                    #self.tc14.AppendText(ele)
                    #self.tc14.SetSelection(-1, -1)
                    #self.tc14.SetAdditionalSelBackground(wx.GREEN)
                    #self.tc14.ShowPosition(10)
                    #self.tc14.SetSelBackground(1, wx.GREEN)
                    #self.tc14.AddSelection(i, i+1)
                    #self.tc14.SetAdditionalSelBackground(wx.RED)                    
                #elif ele == "T":
                    #self.tc14.AppendText(ele)
                    #self.tc14.SetSelection(-1, -1)
                    #self.tc14.SetAdditionalSelBackground(wx.GREEN)
                    #self.tc14.ShowPosition(10)
                    #self.tc14.SetSelBackground(1, wx.GREEN)
                    #self.tc14.AddSelection(i, i+1)
                    #self.tc14.SetAdditionalSelBackground(wx.YELLOW)                                        
            #for mol in pybel.readfile( "mol", sdf_fileName_st):
                #mol_st = str(mol)
                #self_file2 = "%s\n" % (self_file)
                #mol_st22 = mol_st.rstrip(self_file2)
                #mol_st2 = mol_st22.rstrip("\t")
                #mol_st2,filename = mol_st.split("\t")
                #self.tc2.SetValue(mol_st2)

    def OnSubmit1(self, event):
        i_file = str(self.fileName2)
        f_code = str(self.tc12.GetValue())

        flist = os.listdir('C:\SSSCview\FASTA_ssch_database')

        os.chdir('C:\SSSCview')
        fc = open('Fragment_search.txt', 'w')
        fc.write('')
        fc.close()

        n = 0
        for j, elemlist in enumerate(flist):
            os.chdir('C:\SSSCview\FASTA_ssch_database')
            fb = open(elemlist, 'r')
            datab1 = fb.readlines()
            fb.close()
            datab2 = datab1[2]

            if f_code in datab2:
                os.chdir('C:\SSSCview')
                fc = open('Fragment_search.txt', 'a')
                value1 = str(elemlist)
                fc.write(value1)
                fc.write('\n')
                fc.close()
                n = n + 1                
            else:
                continue
            
        self.tc13.AppendText(str(n))
        
    def OnQuit(self, event):
        self.Close()
        
    def OnPDB_MMCIF_GZ_to_ssc(self, event):
        mydir = "C:/SSSCview"
        pydir = "%s/Python" % (mydir)
        os.chdir(pydir)
        os.system("python.exe SSSC_MMCIF_GZ_py37_3_03.py")

    def OnPDB_to_ssc(self, event):
        mydir = "C:/SSSCview"
        pydir = "%s/Python" % (mydir)
        os.chdir(pydir)
        os.system("python.exe SSSC_py37_2_03.py")

    def OnAlignment(self, event):
        mydir = "C:/SSSCview"
        pydir = "%s/Python" % (mydir)
        os.chdir(pydir)
        os.system("python.exe Alignment_input_1_0_2.py")
        
    def OnSSC_to_seq(self, event):
        mydir = "C:/SSSCview"
        pydir = "%s/Python" % (mydir)
        os.chdir(pydir)
        os.system("python.exe ssc_to_seq_1_0_3.py")

    def OnSSSC_to_docx(self, event):
        mydir = "C:/SSSCview"
        pydir = "%s/Python" % (mydir)
        os.chdir(pydir)
        os.system("python.exe SSSC_docx_PyMOL_1_12.py")

    def OnT_characterization(self, event):
        i_file = str(self.fileName2)
        
        class MenuExample1(wx.Frame):
            def __init__(self, parent, id, title):
                wx.Frame.__init__(self, parent, id, title, size=(600, 180))

                menubar = wx.MenuBar()
                file = wx.Menu()
                #file.Append(-1, '&SDF_File_Open')
                #file.AppendSeparator()
                #file.Append(10, '&Open')
                #file.AppendSeparator()        
                quit = wx.Menu()
                quit = wx.MenuItem(file, 1, '&Quit\tCtrl+Q')
                file.Append(quit)
                #file.AppendItem(quit)

                #self.Bind(wx.EVT_MENU, self.OnOpen, id=-1)
                #self.Bind(wx.EVT_MENU, self.OnOpen1, id=10)
                self.Bind(wx.EVT_MENU, self.OnQuit1, id=1)

                menubar.Append(file, '&File')
                self.SetMenuBar(menubar)

                panel = wx.Panel(self, -1)

                font = wx.SystemSettings.GetFont(wx.SYS_SYSTEM_FONT)
                #font = wx.SystemSettings_GetFont(wx.SYS_SYSTEM_FONT)
                font.SetPointSize(10)
                font.SetFamily(wx.FONTFAMILY_MODERN)

                vbox = wx.BoxSizer(wx.VERTICAL)

                vbox.Add((-1, 10))

                hbox3 = wx.BoxSizer(wx.HORIZONTAL)
                self.el2 = []
                #self.el2 = ""
                element_array2 = (self.el2)
                self.tc2 = wx.ComboBox(panel, -1, "Select One", 
                choices = element_array2, style = wx.CB_DROPDOWN)
                hbox3.Add(self.tc2, 1, wx.EXPAND)
                vbox.Add(hbox3, 1, wx.LEFT | wx.RIGHT | wx.EXPAND, 10)
                
                vbox.Add((-1, 10))
                
                hbox5 = wx.BoxSizer(wx.HORIZONTAL)
                btn1 = wx.Button(panel, -1, 'OK', size=(70, 30))
                hbox5.Add(btn1, 0)
                #btn1_2 = wx.Button(panel, -1, 'Tight', size=(70, 30))
                #hbox5.Add(btn1_2, 0, wx.LEFT | wx.BOTTOM, 5)
                #btn1_3 = wx.Button(panel, -1, 'Complete', size=(70, 30))
                #hbox5.Add(btn1_3, 0, wx.LEFT | wx.BOTTOM, 5)
                btn2 = wx.Button(panel, -1, 'Close', size=(70, 30))
                hbox5.Add(btn2, 0, wx.LEFT | wx.BOTTOM, 5)
                vbox.Add(hbox5, 0, wx.ALIGN_RIGHT | wx.RIGHT, 10)

                btn1.Bind(wx.EVT_BUTTON, self.OnSubmit2, id=-1)
                #btn1_2.Bind(wx.EVT_BUTTON, self.OnSubmit3, id=-1)
                #btn1_3.Bind(wx.EVT_BUTTON, self.OnSubmit4, id=-1)
                btn2.Bind(wx.EVT_BUTTON, self.OnQuit1, id=-1)
                
                panel.SetSizer(vbox)
                self.Centre()
                self.Show(True)

                self.CreateStatusBar()
                self.SetStatusText("Code T Characterization")
                self.GetStatusBar().SetBackgroundColour(None)

                self_file = i_file[:-4]
                mchain_file = "%s_mchain.txt" % (self_file)

                mydir = "C:\SSSCview"
                mchaindir = "%s/ConfCode_mchain" % (mydir)
                
                os.chdir(mchaindir)
                fb = open(mchain_file, 'r')
                datab1 = fb.readlines()
                fb.close()
                
                for i, line in enumerate(datab1):
                    #self.tc2.Append(line)
                    if "T, " in line:
                        self.tc2.Append(line)

            def OnSubmit2(self, event):
                def conf_code(self):
                    code_letter ={'A':'4c', 'B':'2a', 'C':'2c', 'D':'5a', 'E':'5c', \
                             'F':'1a', 'G':'1c', 'H':'6a', 'I':'6c', 'J':'3a', 'K':'3c', \
                             'L':'4a'}
                    # ConfCode
                    pi=3.141592653589793115997963468544
                    pi1=pi/6
                    pi2=2*pi/6
                    pi3=3*pi/6
                    pi4=4*pi/6
                    pi5=5*pi/6
                    mpi=-1*pi
                    mpi1=-1*pi/6
                    mpi2=-2*pi/6
                    mpi3=-3*pi/6
                    mpi4=-4*pi/6
                    mpi5=-5*pi/6
                    cc_arith = ""
                    cc = ""
                    # Decision of ConfCode
                    if self>=0 and self<pi1:
                        cc_arith = 'A'
                        cc = "%s" % (code_letter[cc_arith])
                    elif self>=pi1 and self<pi2:
                        cc_arith = 'B'
                        cc = "%s" % (code_letter[cc_arith])
                    elif self>=pi2 and self<pi3:
                        cc_arith = 'C'
                        cc = "%s" % (code_letter[cc_arith])
                    elif self>=pi3 and self<pi4:
                        cc_arith = 'D'
                        cc = "%s" % (code_letter[cc_arith])
                    elif self>=pi4 and self<pi5:
                        cc_arith = 'E'
                        cc = "%s" % (code_letter[cc_arith])
                    elif self>=pi5 and self<=pi:
                        cc_arith = 'F'
                        cc = "%s" % (code_letter[cc_arith])
                    elif self>mpi and self<=mpi5:
                        cc_arith = 'G'
                        cc = "%s" % (code_letter[cc_arith])
                    elif self>mpi5 and self<=mpi4:
                        cc_arith = 'H'
                        cc = "%s" % (code_letter[cc_arith])
                    elif self>mpi4 and self<=mpi3:
                        cc_arith = 'I'
                        cc = "%s" % (code_letter[cc_arith])
                    elif self>mpi3 and self<=mpi2:
                        cc_arith = 'J'
                        cc = "%s" % (code_letter[cc_arith])
                    elif self>mpi2 and self<=mpi1:
                        cc_arith = 'K'
                        cc = "%s" % (code_letter[cc_arith])
                    elif self>mpi1 and self<0:
                        cc_arith = 'L'
                        cc = "%s" % (code_letter[cc_arith])
                    else:
                        cc_arith = None
                        cc = None
                    return cc

                def _test_homology(self1, self2):
                    """Return 1 if structural homology between self1 snd self2 exists."""
                    code_list =['4c', '2a', '2c', '5a', '5c', \
                                '1a', '1c', '6a', '6c', '3a', '3c', \
                                '4a']
                    code_3letter_1 ={'A':'LAB', 'B':'ABC', 'C':'BCD', \
                                    'D':'CDE', 'E':'DEF', 'F':'EFG', \
                                    'G':'FGH', 'H':'GHI', 'I':'HIJ', \
                                    'J':'IJK', 'K':'JKL', 'L':'KLA'}
                    code_3letter_2 ={'4c':'A', '2a':'B', '2c':'C', '5a':'D', '5c':'E', \
                                    '1a':'F', '1c':'G', '6a':'H', '6c':'I', '3a':'J', '3c':'K', \
                                    '4a':'L'}
                    if self1 in code_list:
                        code1 = "%s" % (code_3letter_2[self1])
                    else:
                        return 0
                    code3_1 = code_3letter_1[code1]
                    lcode3_1 = list(code3_1)
                    if self2 in code_list:
                        code2 = "%s" % (code_3letter_2[self2])
                    else:
                        return 0
                    code3_2 = code_3letter_1[code2]
                    lcode3_2 = list(code3_2)
                    for code_index, code in enumerate(lcode3_1) :
                        if code in lcode3_2 :
                            return 1
                        else:
                            if code_index == 2 :
                                return 0
                            else:
                                continue

                os.chdir('C:\\SSSCview')
                fc = open('ConfCode_T_list.txt', 'w')
                fc.write('')
                fc.close()

                flist1 = os.listdir('C:\\SSSCview\ConfCode_mchain')

                ele = str(self.tc2.GetValue())
                if "T, " in ele:
                    ele_s = ele.strip('\n')
                    res_name,sscHST,ccphi,ccpsi,ccomega = ele_s.split(", ")

                for i, elemlist in enumerate(flist1):
                    os.chdir('C:\\SSSCview\ConfCode_mchain')
                    fname1 = "C:\\SSSCview\ConfCode_mchain\%s" % (str(elemlist))
                    fname2 = elemlist[:-4]
                    fa = open(str(fname1), 'r')
                    dataa1 = fa.readlines()
                    fa.close()
                    
                    for j, ele2 in enumerate(dataa1):
                        if j != 0:
                            try:
                                ele2_s = ele2.strip('\n')
                                res_name2,sscHST2,ccphi2,ccpsi2,ccomega2 = ele2_s.split(", ")
                                if "T" in sscHST2:
                                    Hm = 1
                                    Hb1 = _test_homology(ccphi, ccphi2)
                                    Hb2 = _test_homology(ccpsi, ccpsi2)
                                    Hb3 = _test_homology(ccomega, ccomega2)
                                    Hm = Hm * Hb1 * Hb2 * Hb3
                                    if Hm == 1:
                                        os.chdir('C:\\SSSCview')
                                        fc = open('ConfCode_T_list.txt', 'a')
                                        fc.write(str(fname2))
                                        fc.write(', ')
                                        fc.write(ele2)
                                        fc.close()
                            except:
                                res2 = 0

            def OnQuit1(self, event):
                self.Close()

        app1 = wx.App()
        MenuExample1(None, -1, 'T_characterization')
        app1.MainLoop()

    def OnFuzzy_search(self, event):
        i_file = str(self.fileName2)
        file = open(os.path.join(self.dirName2, self.fileName2), 'r')
        dataa1 = file.readlines()
        file.close()
        
        #seq1 = list(dataa1[1])
        #ssc1 = list(dataa1[2])
        seq1 = dataa1[1]
        ssc1 = dataa1[2]
        dataa4 = [ssc1[i: i+20] for i in range(0, len(ssc1), 20)]
        dataa5 = [ssc1[i: i+20] for i in range(10, len(ssc1), 20)]
        #print dataa4
        #print dataa5

        os.chdir('C:\\SSSCview')
        fc = open('Struct_homology_value.txt', 'w')
        fc.write('')
        fc.close()

        flist = os.listdir('C:\SSSCview\FASTA_ssch_database')

        for j, elemlist in enumerate(flist):
            os.chdir('C:\SSSCview\FASTA_ssch_database')
            fb = open(elemlist, 'r')
            datab1 = fb.readlines()
            fb.close()
            #seq2 = list(datab1[1])
            #ssc2 = list(datab1[2])
            seq2 = datab1[1]
            ssc2 = datab1[2]   
            datab4 = [ssc2[i: i+20] for i in range(0, len(ssc2), 20)]
            datab5 = [ssc2[i: i+20] for i in range(10, len(ssc2), 20)]

            s1 = SequenceMatcher(None, dataa4, datab4)
            ratio1 = s1.ratio()
            ratioall = [ratio1]

            for i, elem in enumerate(dataa4):
                if len(elem) < 4:
                    continue
                else:
                    eleml = len(elem)
                    if elem.count('S') >= 18:
                        continue
                    else:
                        if elem.count('H') >= 18:
                            continue
                        else:
                            for j, elem2 in enumerate(datab4):
                                s2 = SequenceMatcher(None, elem, elem2)
                                ratio2 = s2.ratio()
                                ratioall.append(ratio2)

            ratiomax = max(ratioall)

            ratioall2 = [ratio1]

            for i, elem in enumerate(dataa5):
                if len(elem) < 4:
                    continue
                else:
                    elem2l = len(elem)
                    if elem.count('S') >= 18:
                        continue
                    else:
                        if elem.count('H') >= 18:
                            continue
                        else:
                            for j, elem2 in enumerate(datab5):
                                s3 = SequenceMatcher(None, elem, elem2)
                                ratio3 = s3.ratio()
                                ratioall2.append(ratio3)

            ratiomax2 = max(ratioall2)

            os.chdir('C:\\SSSCview')
            fc = open('Struct_homology_value.txt', 'a')
            value1 = str(elemlist)
            value2 = str(ratio1)
            value3 = str(ratiomax)
            value4 = str(ratiomax2)
            fc.write(value1)
            fc.write('\t')
            fc.write(value2)
            fc.write('\t')
            fc.write(value3)
            fc.write('\t')
            fc.write(value4)
            fc.write('\n')
            fc.close()
         

app = wx.App()
MenuExample(None, -1, 'SSSCview')
app.MainLoop()
        
